function vout = OLDtsmovavg(vin, mode, varargin)
%TSMOVAVG calculates the (weighted) moving average of vector of data.
%
%   Syntax: VO = tsmovavg(VI, 's', LEAD, LAG, DIM)      => SIMPLE, s
%           VO = tsmovavg(VI, 'e', TIMEPER, DIM)        => EXPONENTIAL, e
%           VO = tsmovavg(VI, 't', NUMPER, DIM)         => TRIANGULAR, t
%           VO = tsmovavg(VI, 'w', WEIGHTS, PIVOT, DIM) => WEIGHTED, w
%           VO = tsmovavg(VI, 'm', NUMPER, DIM)         => MODIFIED, m
%
%   VI is assume to be a row vector or row-oriented matrix i.e. each row 
%   is a set of observation.  VO is identical in form to VI.  Thus, the 
%   default is DIM = 2,  However, if DIM is 1, VI will be assumed to be 
%   a column vector of a column-oriented matrix (each column is a set of 
%   observations).
%
%   Simple Moving Average:
%   LEAD and LAG are parameters indicating the number of previous and 
%   following data points to be used in conjunction with the current data 
%   point when calculating the moving average.  For example, if you want
%   to calculate a 5-day moving average, with the current data in the
%   middle, you set LEAD and LAG to 2 (2 + 1 + 2 = 5).
%
%   Exponential Moving Average:
%   Exponential moving average is a weighted moving average is with the 
%   assigned weights decreasing exponentially as go further into the past. 
%   The following method describes simple exponential smoothing.  a (alpha) 
%   is a smoothing constant, the most recent value of the time series is 
%   weighted by a , the next most recent value is weighted by a (1-a), the 
%   next value by a (1-a)^2, and so forth.  Here, a (alpha) is calculated
%   using 2/(TIMEPER+1), or 2/(Windows_size+1).
%
%   Triangular Moving Average:
%   Triangular moving average is nothing else but double-smoothing the data.
%   It calculates the first simple moving average with window width of
%   NUMPER/2.  If NUMPER is an odd number, round up (NUMPER/2) and use it to
%   calculate both the first and the second moving average.  The second 
%   moving average is just a simple moving average of the first moving 
%   average.  Now, if NUMPER is an even number, calculate the first moving
%   average using width (NUMPER/2) and the second moving average using width
%   (NUMPER/2)+1.
%
%   Weighted Moving Average:
%   It is calculated by supplying the weights, WEIGHTS, for each element
%   in the moving window.  The length of the weight vector determines
%   the size of the window.  The PIVOT point is where the average should
%   be placed.  For example, if WEIGHTS = [1 1 1 1 1] and PIVOT = 3, it 
%   will calculate a simple moving average where they are averages of 2 
%   values prior to the current value, the current value itself, and 2 
%   values after the current value.
%
%   Modified Moving Average:
%   The Modified MA is calculated almost similarly to Simple MA.  The first
%   moving average value, VO(NUMPER), is calculated by simply averaging the
%   past NUMPER VI's.  The rest of the moving average values are calculated 
%   by adding to the previous MA value the current data point divided by 
%   NUMPER and subtracting the previous MA divided by NUMPER.  MA values
%   prior to NUMPER-th value are copies of the data values.
%
%   See also MEAN, PERAVG.
%

%
%   Reference: Achelis, Steven B., Technical Analysis From A To Z,
%              Second Printing, McGraw-Hill, 1995, pg. 184-192
%

%   Author: P. N. Secakusuma, 01-08-98
%   Copyright (c) 1995-2000 The MathWorks, Inc.
%   $Revision: 1.10 $  $Date: 2000/04/05 19:32:56 $

if nargin == 1,
   error('Please specify MODE!');
end

switch lower(mode(1)),
case 's',
   if nargin == 3,
      lead = varargin{1};
      lag = 0;
   elseif nargin == 4,
      lead = varargin{1};
      lag = varargin{2};
   elseif nargin == 5,
      lead = varargin{1};
      lag = varargin{2};
      dim = varargin{3};
      if dim == 1,   % If input is column-oriented, transpose.
         vin = vin';
      end
   else,
      error('Must have at least LEAD for simple moving average!');
   end
   vout = zeros(size(vin, 1), size(vin, 2));
   
   for idx = 1:lead,
      vout(:, idx) = mean(vin(:, 1:idx), 2);
   end
   for idx = lead+1:length(vin)-lag,
      vout(:, idx) = mean(vin(:, idx-lead:idx+lag), 2);
   end
   if lag,
      for idx = length(vin)-lag+1:length(vin),
         vout(:, idx) = mean(vin(:, length(vin)-lag+1:end), 2);
      end
   end

case 'e',
   if nargin == 3,
      timeper = varargin{1};
   elseif nargin == 4,
      timeper = varargin{1};
      dim = varargin{2};
      if dim == 1,   % If input is column-oriented, transpose.
         vin = vin';
      end
   elseif nargin < 3,
      timeper = 5;
   else,
      error('Must have only TIMEPER for exponential moving average!');
   end
   exppct = 2/(timeper+1);
   vout = vin;
   
   for idx = 2:length(vin),
      vout(:, idx) = exppct.*vin(:, idx) + (1-exppct).*vout(:, idx-1);
   end
   
case 't',
   if nargin == 3,
      numper = varargin{1};
   elseif nargin == 4,
      numper = varargin{1};
      dim = varargin{2};
      if dim == 1,   % If input is column-oriented, transpose.
         vin = vin';
      end
   elseif nargin < 3,
      numper = 5;
   else,
      error('Must have only NUMBER for triangular moving average!');
   end
   maper = ceil((numper+1)/2);
   mamaper = maper;
   vout = vin;
   
   for jdx = 1:size(vin, 1)
      mavin   = conv(vin(jdx, :),   (1./(maper*ones(1, maper))));
      mamavin(jdx, :) = conv(mavin, (1./(mamaper*ones(1, maper))));
%      mavin = tsmovavg(vin(jdx, :), 's', maper-1);
%      mamavin = tsmovavg(mavin, 's', mamaper-1);
   end
   
   overshoots = floor(numper / 2);
   vout = mamavin(:, overshoots+1:end-overshoots);
   
case 'w',
   if nargin == 4,
      wts = varargin{1};
      pvt = varargin{2};
      if length(pvt) ~= 1,
         error('PIVOT point must be a scalar index!');
      end
   elseif nargin == 5,
      wts = varargin{1};
      pvt = varargin{2};
      if length(pvt) ~= 1,
         error('PIVOT point must be a scalar index!');
      end
      dim = varargin{3};
      if dim == 1,   % If input is column-oriented, transpose.
         vin = vin';
      end
   else
      error('Must have WEIGHTS and PIVOT point!');
   end
   wtsum = sum(wts);
   vout = vin;
   
   for idx = pvt:length(vin)-(length(wts)-pvt),
      vout(:, idx) = sum(repmat(wts, size(vin, 1), 1).*vin(:, idx-(pvt-1):idx+(length(wts)-pvt)), 2) / wtsum;
   end
   
case 'm',   % Modified Moving Average.   
   if nargin == 3,
      numper = varargin{1};
   elseif nargin == 4,
      numper = varargin{1};
      dim = varargin{2};
      if dim == 1,   % If input is column-oriented, transpose.
         vin = vin';
      end
   elseif nargin < 3,
      numper = 5;
   else,
      error('Must have only NUMBER for triangular moving average!');
   end
   vout = vin;      
   
   vout(:, numper) = mean(vin(:, 1:numper), 2);
   for idx = numper+1:size(vin, 2),
      vout(:, idx) = vout(:, idx-1) + (vin(:, idx)-vout(:, idx-1))./numper;
   end
      
otherwise,
   error('Invalid mode for moving average calculation!');
   
end

% Transpose output, if appropriate.
if exist('dim', 'var') & dim == 1,   % If input is column-oriented, transpose.
   vout = vout';
end

return

