function [Xn,Yn,dXn,dYn,ddXn,ddYn]=reconst_angle_der(l,theta,Xorg,Yorg,dtheta,dXorg,dYorg,ddtheta,ddXorg,ddYorg)

% Dtermination des coordonnes (trois premires drives) d'un kinogramme  partir des angles articulaires
%
% *********************************************************
%  [Xn,Yn,dXn,dYn,ddXn,ddYn]=reconst_angle_der(l,theta,Xorg,Yorg,dtheta,dXorg,dYorg,ddtheta,ddXorg,ddYorg) : 
%
%	* Variables d'entres :
%     *  l :  tableau de type (1,p-1) reprsentant les longueurs 
%     * theta : tableau de type (n+1,p-1) o pour tout i dans {0,...,n}, 
%                                         pour tout j dans {1,...,p-1},
%              theta(i+1,j) est l'angle articulaire (A{j-1}_A_{j},A{j}_A_{j+1})
%               l'instant t_i=i/fe (fe : frquence d'acquisition)
%     * Xorg : tableau de taille (n+1,1) o 
%              o pour tout i dans {0,...,n}, 
%              CX(i+1) est l'abscisse de A_1  l'instant t_i=i/fe
%     * Yorg : tableau de taille (n+1,1) o 
%              o pour tout i dans {0,...,n}, 
%              CX(i+1) est l'ordonne de A_1  l'instant t_i=i/fe
%	* Variables d'entres optionnelles (dans l'ordre) :
%     * dtheta : tableau de type (n+1,p-1) o pour tout i dans {0,...,n}, 
%                                         pour tout j dans {1,...,p-1},
%              theta(i+1,j) est la drive de l'angle articulaire (A{j-1}_A_{j},A{j}_A_{j+1})
%               l'instant t_i=i/fe
%     * dXorg : tableau de taille (n+1,1) o 
%              o pour tout i dans {0,...,n}, 
%              dXorg(i+1) est la drive de l'abscisse de A_1  l'instant t_i=i/fe
%     * dYorg : tableau de taille (n+1,1) o 
%              o pour tout i dans {0,...,n}, 
%              dYorg(i+1) est la drive de l'ordonne de A_1  l'instant t_i=i/fe
%     * ddtheta : tableau de type (n+1,p-1) o pour tout i dans {0,...,n}, 
%                                         pour tout j dans {1,...,p-1},
%              theta(i+1,j) est la drive seconde de l'angle articulaire (A{j-1}_A_{j},A{j}_A_{j+1})
%               l'instant t_i=i/fe
%     * ddXorg : tableau de taille (n+1,1) o 
%              o pour tout i dans {0,...,n}, 
%              ddXorg(i+1) est la drive seconde de l'abscisse de A_1  l'instant t_i=i/fe
%     * ddYorg : tableau de taille (n+1,1) o 
%              o pour tout i dans {0,...,n}, 
%              ddYorg(i+1) est la drive seconde de l'ordonne de A_1  l'instant t_i=i/fe
%
%  * Variables de sortie :
%     *  Xn :  tableau de type (n+1,p) o pour tout i dans {0,...,n}, 
%                                         pour tout j dans {1,...,p},
%              Xn(i+1,j) est l'abscisse de A_j  l'instant t_i=i/fe
%     *  Yn :  tableau de type (n+1,p) o pour tout i dans {0,...,n}, 
%                                         pour tout j dans {1,...,p},
%              Yn(i+1,j) est l'ordonne de A_j  l'instant t_i=i/fe
%  * Variables de sortie (optionnelle dans l'ordre)
%     *  dXn :  tableau de type (n+1,p) o pour tout i dans {0,...,n}, 
%                                         pour tout j dans {1,...,p},
%               dXn(i+1,j) est la drive de l'abscisse de A_j  l'instant t_i=i/fe
%     *  dYn :  tableau de type (n+1,p) o pour tout i dans {0,...,n}, 
%                                         pour tout j dans {1,...,p},
%               dYn(i+1,j) est la drive de l'ordonne de A_j  l'instant t_i=i/fe
%     *  ddXn :  tableau de type (n+1,p) o pour tout i dans {0,...,n}, 
%                                         pour tout j dans {1,...,p},
%               dXn(i+1,j) est la drive seconde de l'abscisse de A_j  l'instant t_i=i/fe
%     *  ddYn :  tableau de type (n+1,p) o pour tout i dans {0,...,n}, 
%                                         pour tout j dans {1,...,p},
%               dYn(i+1,j) est la drive seconde de l'ordonne de A_j  l'instant t_i=i/fe
%
%
% ************ Fonctions auxiliaires utilises ************
%
%     cumsumsym2 (pour des calculs symboliques uniquement)
%
% *********************************************************
%
% 2007 by          Jrme BASTIEN
%                  Universit Claude Bernard Lyon I, UFRSTAPS, Laboratoire CRIS, Villeurbanne
%                  E-Mail : jerome.bastien@univ-lyon1.fr

nin=nargin;
nout=nargout;
if ~(((nin==4)&(nout==2))|((nin==7)&(nout==4))|((nin==10)&(nout==6)))
    error('le nombre d''argument de sortie n''est pas correct par rapport au nombre d''arguments d''entree');
end
jr=length(l);
[q,j]=size(theta);
if (q==0)
   error('ATTENTION : pas de temps !'); 
end
if (j~=jr)
   error('Les deux tableaux n''ont pas la taille adquate'); 
end
resX=zeros(j+1,q);
resY=zeros(j+1,q);
if (nout>=4)
    dresX=zeros(j+1,q);
    dresY=zeros(j+1,q);
end
if (nout>=6)
    ddresX=zeros(j+1,q);
    ddresY=zeros(j+1,q);
end
if (j~=0)
    if (isnumeric(theta))
        auxitheta=cumsum(theta,2);
        c=cos(auxitheta);
        s=sin(auxitheta);
        onel=ones(q,1)*l;
        resX=(cumsum(c.*onel,2)).';
        resY=(cumsum(s.*onel,2)).';
        if (nout>=4)
            dauxitheta=cumsum(dtheta,2);
            dresX=-(cumsum(dauxitheta.*s.*onel,2)).';
            dresY=(cumsum(dauxitheta.*c.*onel,2)).';
        end
        if (nout>=6)
            ddauxitheta=cumsum(ddtheta,2);
            ddresX=-(cumsum((ddauxitheta.*s+dauxitheta.^2.*c).*onel,2)).';
            ddresY=(cumsum((ddauxitheta.*c-dauxitheta.^2.*s).*onel,2)).';
        end
    else
        auxitheta=cumsumsym2(theta);
        c=cos(auxitheta);
        s=sin(auxitheta);
        onel=ones(q,1)*l;
        resX=(cumsumsym2(c.*onel)).';
        resY=(cumsumsym2(s.*onel)).';
        if (nout>=4)
            dauxitheta=cumsumsym2(dtheta);
            dresX=-(cumsumsym2(dauxitheta.*s.*onel)).';
            dresY=(cumsumsym2(dauxitheta.*c.*onel)).';
        end
        if (nout>=6)
            ddauxitheta=cumsumsym2(ddtheta);
            ddresX=-(cumsumsym2((ddauxitheta.*s+dauxitheta.^2.*c).*onel)).';
            ddresY=(cumsumsym2((ddauxitheta.*c-dauxitheta.^2.*s).*onel)).';
        end
    end
    resX=[zeros(1,q);resX];
    resY=[zeros(1,q);resY];
    if (nout>=4)
        dresX=[zeros(1,q);dresX];
        dresY=[zeros(1,q);dresY];
    end
    if (nout>=6)
        ddresX=[zeros(1,q);ddresX];
        ddresY=[zeros(1,q);ddresY];
    end
end
Xn=resX.'+Xorg(:,ones(1,j+1));
Yn=resY.'+Yorg(:,ones(1,j+1));
if (nout>=4)
    dXn=dresX.'+dXorg(:,ones(1,j+1));
    dYn=dresY.'+dYorg(:,ones(1,j+1));
end
if (nout>=6)
    ddXn=ddresX.'+ddXorg(:,ones(1,j+1));
    ddYn=ddresY.'+ddYorg(:,ones(1,j+1));
end